/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.cube;
import scalasca.cubex.cube.errors.*;
import scalasca.cubex.cube.services.*;
import java.lang.*;
import java.util.*;
import scalasca.cubex.cube.datalayout.*;
import scalasca.cubex.cube.datalayout.data.*;
import scalasca.cubex.cube.datalayout.data.value.*;

/*
 * Class , delivers file structure of CubeX
 * 
 */
public class InclusiveMetric extends Metric
{
	public InclusiveMetric( int _id,   Metric _parent )
	{
		super(_id,  _parent);
	}

	public void createEnumeration(ArrayList<scalasca.cubex.cube.Cnode> root_cnodes)
	{
		enumeration = new CnodesWideEnumeration(root_cnodes);
	}



	public Value get_sev(Cnode cnode, CubeCalculationFlavour cf, Location loc, CubeCalculationFlavour lf) throws NotEnumeratedCnodeException
	{
		Value _tmp = metric_value.clone();
		int CID = enumeration.getCID(cnode);
		Value value = (matrix!= null)?matrix.getValue(CID, loc.getId()).copy():this.metric_value.clone();
		if (cf == CubeCalculationFlavour.EXCLUSIVE )
		{
			for (scalasca.cubex.cube.Vertex _cnode : cnode.getAllChildren())
			{
				_tmp.addValue(get_sev((scalasca.cubex.cube.Cnode)_cnode, CubeCalculationFlavour.INCLUSIVE, loc, lf));
			}
			value.subtractValue(_tmp);
		}
		return value;
	}



	public Value get_sev(Cnode cnode, CubeCalculationFlavour cf) throws NotEnumeratedCnodeException
	{
        Value _tmp = metric_value.clone();
        int CID = enumeration.getCID(cnode);
        Value value = (matrix!= null)?matrix.sumRow(CID).copy():this.metric_value.clone();
        if (cf == CubeCalculationFlavour.EXCLUSIVE )
        {
            for (scalasca.cubex.cube.Vertex _cnode : cnode.getAllChildren())
            {
                _tmp.addValue(get_sev((scalasca.cubex.cube.Cnode)_cnode, CubeCalculationFlavour.INCLUSIVE));
            }
            value.subtractValue(_tmp);
        }
        return value;
	}


}
