/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.cube.datalayout.data;
import scalasca.cubex.cube.datalayout.data.value.*;
import scalasca.cubex.cube.errors.*;
import scalasca.cubex.cube.services.transformation.*;
import java.lang.*;
import java.util.*;
import java.io.*;
/*
 * Class , delivers file structure of CubeX
 * 
 */
public class DataRows
{
	Value value ;
	Value[] data = null;
	public DataRows(byte[] _data, Value _value, Endianess _endianess) throws BadDataException
	{
		loadData(_data, _value, _endianess);
	};



	protected void loadData(byte[] _data, Value _value, Endianess _endianess)  throws BadDataException
	{
		value = _value;
		byte[] correct_data = null;
		try
		{
			correct_data = value.correctEndianess(_data, _endianess);
			data = getRowOfValues(correct_data);
		}catch (Exception e)
		{
			throw new BadDataException("Error in loading uncompressed data: " + e.toString());
		}
	}



	protected Value[] getRowOfValues(byte[] _data)  throws IOException
	{

		DataInputStream in = new DataInputStream( new ByteArrayInputStream(_data));
		Value[] row = new Value[ _data.length / value.size()];
		for (int i = 0; i< row.length; i++)
		{
			row[i] = value.clone(in);
		}
		return row;
	}



	public void print()
		{
			System.out.println("======== Raw rows of data ============");
			System.out.println("======== "+data.length + " bytes ============");
			
			for (int i=0; i< data.length; i++)
			{
 				System.out.print(data[i].toString() + " ");
			}
			System.out.println();

		}

	public Value getValue() { return value; }



	public Value[] getRow(int position, int number)
	{
		Value[] row = new Value[number];
		System.arraycopy(data, position, row, 0, number);
		return row;
	}

	public  void removeRowsFromMemory()
	{
		// danger... can cause memory errors
		data = null;
	}


}
