/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.cube.services;
import java.lang.*;
/*

 * Class , delivers file structure of CubeX
 * 
 */
public class ProgressReportContext
{
	private ProgressReportContext previous = null;
	protected double achieved=0;     // acomplished progress
	protected double step=0;         // size of current step
	protected String progress_message; // actual progress message

	protected Object subActor = null;           // means me is the last in chain

	private int level=0;

	public ProgressReportContext()
	{
		previous =null;
		achieved = 0;
		step = 0;
		level = 0;
	}
	public ProgressReportContext(ProgressReportContext _previous)
	{
		achieved = 0;
		step = 0;
		previous =_previous;
		if (previous != null)
			level = previous.getLevel() + 1;
	}
		
	public void setNextStep(double v)
	{
		subActor = null; // next step assumes that actor finished its job
		step=v;
	}
	public void setAchievedStep()
	{
		subActor = null; // next step assumes that actor finished its job
		achieved += step;
	}
	public ProgressReportContext createSubProgressReport()
	{
		return  new ProgressReportContext(this);
	}

	public int getLevel() { return level;}

	public void setProgressMessage(String _status) { progress_message = _status; }

	public String getProgressMessageChain()
	{
		String _to_return = progress_message;
		String ident="";
		for (int i=0; i<level; i++)
			ident = ident.concat("        ");
		if (previous != null)
			_to_return= previous.getProgressMessageChain() + "\n"+ident+ "->"+_to_return;
		return _to_return;
	}

	public String getProgressMessage()
	{
		if (subActor != null && (subActor instanceof ProgressReportInterface))
			return ((ProgressReportInterface)(subActor)).getProgressMessage();  // actual message (full is caculated bz the last guy in chain)

		return getProgressMessageChain();
	}



	public double getProgress() // should return from 0 to 1 progress of its step
	{
		double _progress = 0.;
		if (subActor != null && (subActor instanceof ProgressReportInterface))
		{
			double __progress = ((ProgressReportInterface)(subActor)).getProgress();
			_progress = this.achieved + this.step * __progress;  // actual message (full is caculated bz the last guy in chain)
			return _progress;
		}
		return achieved;
	}

	public ProgressReportContext getPreviousContext() { return previous; }

	public void setSubActor(Object _actor)
		{
			subActor = _actor;
			((ProgressReportInterface)subActor).setActiveProgressReportContext(this.createSubProgressReport());
		}
	public void setNoSubActor()
		{
			subActor = null;
		}

}
